@@ -93,5 +93,16 @@ C(super7)
 C(swdistmx)
 C(transalnref)
 C(mega_msas)
+C(masm_train)
+C(sw)
+C(swmasm)
+C(swmasm_seq)
+C(masm_stats)
+C(mega2)
+C(test_sw_aa)
+C(test_sw_mm)
+C(swtest)
+C(swtestmm)
+C(swsimple2)
 
 #undef C
