@@ -11,10 +11,12 @@ public:
 	uint m_SlotCount = 0;
 	vector<vector<uint> > m_HashToSeqIndexes;
 
+	bool m_Disable = false;
+
 public:
 	uint CalcHash(const Sequence *Seq) const;
 	void Clear();
-	void Run(MultiSequence &InputSeqs);
+	void Run(MultiSequence &InputSeqs, bool ShowProgress = true);
 	uint Search(uint SeqIndex) const;
 	void GetUniqueSeqs(MultiSequence &UniqueSeqs);
 	void AddToHash(uint SeqIndex);
@@ -22,4 +24,6 @@ public:
 	void Validate() const;
 	void GetDupeGSIs(vector<uint> &GSIs,
 	  vector<uint> &GlobalRepSeqIndexes) const;
+	void GetRepLabelToDupeLabels(
+	  unordered_map<string, vector<string> > &RepLabelToDupeLabels) const;
 	};
